#!/usr/bin/env python3
# -*- encoding: utf-8; py-indent-offset: 4 -*-
"""
This is the check for IBM 4500 Tape Library
"""
__author__ = "Christian Michaelski"
__license__ = "GPL"
__version__ = "2.0.0p1"
__maintainer__ = "Christian Michaelski"
__email__ = "cmichaelski@conet.de"

from typing import Dict, List, Optional, Tuple, Union, Mapping, Any
from .agent_based_api.v1 import *
from .agent_based_api.v1.type_defs import CheckResult, DiscoveryResult
import pprint

StatusInfo = Tuple[float, Tuple[int, str]]
Parsed = Dict[str, Dict[str, Union[float, StatusInfo]]]

# check states suggested by customer
mapping = {
    "avail" : {
        "1"  : (State.WARN, "other"),
        "2"  : (State.UNKNOWN, "unknown"),
        "3"  : (State.OK, "running full power"),
        "4"  : (State.WARN, "warning"),
        "5"  : (State.WARN, "in test"),
        "6"  : (State.UNKNOWN, "not applicable"),
        "7"  : (State.WARN, "power off"),
        "8"  : (State.WARN, "off line"),
        "9"  : (State.WARN, "off duty"),
        "10" : (State.WARN, "degraded"),
        "11" : (State.WARN, "not installed"),
        "12" : (State.CRIT, "install error"),
        "13" : (State.UNKNOWN, "power save unknown"),
        "14" : (State.OK, "power save low power mode"),
        "15" : (State.OK, "power save standby"),
        "16" : (State.WARN, "power cycle"),
        "17" : (State.WARN, "power save warning"),
        "18" : (State.WARN, "paused"),
        "19" : (State.WARN, "not ready"),
        "20" : (State.WARN, "not configured"),
        "21" : (State.WARN, "quiesced"),
    },
    "status" : {
        "0"     : (State.UNKNOWN, "unknown"),
        "1"     : (State.WARN, "other"),
        "2"     : (State.OK, "ok"),
        "3"     : (State.WARN, "degraded"),
        "4"     : (State.WARN, "stressed"),
        "5"     : (State.WARN, "predictive failure"),
        "6"     : (State.CRIT, "error"),
        "7"     : (State.CRIT, "non-recoverable error"),
        "8"     : (State.WARN, "starting"),
        "9"     : (State.WARN, "stopping"),
        "10"    : (State.WARN, "stopped "),
        "11"    : (State.WARN, "in service"),
        "12"    : (State.UNKNOWN, "no contact"),
        "13"    : (State.UNKNOWN, "lost communication"),
        "14"    : (State.CRIT, "aborted"),
        "15"    : (State.WARN, "dormant"),
        "16"    : (State.CRIT, "supporting entity in error"),
        "17"    : (State.WARN, "completed"),
        "18"    : (State.WARN, "power mode"),
        "19"    : (State.WARN, "dMTF reserved"),
        "32768" : (State.UNKNOWN, "vendor reserved"),\
    }
}

def ibm_tape_library_parse_device_name(name):
    # strange name format:IBM     ULT3580-TD6     00078B5F0F
    return " ".join([part for part in name.split() if part])


def _parse_ibm_tl_media_access_devices(media_access_info, controller_info ):
    parsed = {}
    for ty, name, clean in media_access_info:
        parsed.setdefault(
            ibm_tape_library_parse_device_name(name), {
                "type": {
                    "0": "unknown",
                    "1": "worm drive",
                    "2": "magneto optical drive",
                    "3": "tape drive",
                    "4": "dvd drive",
                    "5": "cdrom drive",
                }[ty],
                "clean": {
                    "0": "unknown",
                    "1": "true",
                    "2": "false",
                }[clean],
            })

    for ctrl_name, ctrl_avail, ctrl_status in controller_info:
        ctrl_name = ibm_tape_library_parse_device_name(ctrl_name)
        if ctrl_name in parsed:
            parsed[ctrl_name]["ctrl_avail"] = ctrl_avail
            parsed[ctrl_name]["ctrl_status"] = ctrl_status

    return parsed


def parse_ibm_tl_4500(string_table: List[type_defs.StringTable]) -> Optional[Parsed]:
    parsed: Parsed = {}
    if not any(string_table): 
        return None

    parsed["changer"] = string_table[0]
    parsed["media"] = _parse_ibm_tl_media_access_devices(string_table[1], string_table[2])
    return parsed


register.snmp_section(
    name="ibm_tl_4500",
    parse_function=parse_ibm_tl_4500,
    fetch=[
        SNMPTree(
            base=".1.3.6.1.4.1.14851.3.1.11.2.1",
            oids=[
            "4",  # SNIA-SML-MIB::changerDevice-ElementName
            "8",  # SNIA-SML-MIB::changerDevice-Availability
            "9",  # SNIA-SML-MIB::changerDevice-OperationalStatus
            ],
        ),
        SNMPTree(
            base=".1.3.6.1.4.1.14851.3.1.6.2.1",
            oids=[
                "2",  # SNIA-SML-MIB::mediaAccessDeviceObjectType
                "3",  # SNIA-SML-MIB::mediaAccessDevice-Name
                "6",  # SNIA-SML-MIB::mediaAccessDevice-NeedsCleaning
            ],
        ),
        SNMPTree(
            base=".1.3.6.1.4.1.14851.3.1.12.2.1",
            oids=[
                "3",  # SNIA-SML-MIB::scsiProtocolController-ElementName
                "6",  # SNIA-SML-MIB::scsiProtocolController-Availability
                "4",  # SNIA-SML-MIB::scsiProtocolController-OperationalStatus
            ],
        ),
    ],
    detect=any_of(
        startswith(".1.3.6.1.2.1.1.2.0", '.1.3.6.1.4.1.2.6.254.1'),
        exists(".1.3.6.1.4.1.14851.*"),
    ),
)


def discover_ibm_tl_4500_changer(section):
    for name, _avail, _status in section['changer']:
        svcname = name.replace("Logical_Library:", "").strip()
        yield Service(item=svcname)

def check_ibm_tl_4500_changer(item: str, params: Mapping[str, Any], section: Parsed) -> CheckResult:
    for name, _avail, _status in section['changer']:
        name = name.replace("Logical_Library:", "").strip()
        if name == item:
            for what, val, text in [("avail", _avail, "Availability"), ("status", _status, "Status")]:
                state, state_readable = mapping[what][val]
                yield Result(state=state, summary=f"{text}: {state_readable}")


register.check_plugin(
    name='ibm_tl_4500_changer',
    sections=['ibm_tl_4500'],
    service_name='Changer device %s',
    check_default_parameters={},
    discovery_function=discover_ibm_tl_4500_changer,
    check_function=check_ibm_tl_4500_changer,
)

def discover_ibm_tl_4500_media(section):
    for device in section['media'].keys():
        yield Service(item=device)

def check_ibm_tl_4500_media(item: str, params: Mapping[str, Any], section: Parsed) -> CheckResult:
    for device in section['media'].keys():
        if item == device:
            data = section['media'][item]
            if data.get("ctrl_avail") and data.get("ctrl_status"):
                for what, val, text in [("avail", data.get("ctrl_avail"), "Availability"), ("status", data.get("ctrl_status"), "Status")]:
                    state, state_readable = mapping[what][val]
                    yield Result(state=state, summary=f"{text}: {state_readable}")
            yield Result(state=State.OK, summary="Type: %s, Needs cleaning: %s" % (data["type"], data["clean"]))


register.check_plugin(
    name='ibm_tl_4500_media',
    sections=['ibm_tl_4500'],
    service_name='Media access device %s',
    check_default_parameters={},
    discovery_function=discover_ibm_tl_4500_media,
    check_function=check_ibm_tl_4500_media,
)
